//  Demo 26: RunnableSquare.java
//
//     A Moveable Square Which Animates Itself.
//     Another Simple Example of Subclassing
//     Illustrates the Runnable Interface
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999

// There is another method for running animation in separate
// Threads which does not require the objects doing the
// animating to be a Threads.  RunnableSquare and the next
// demo illustrate this approach.

// As you probably know, Java does not allow a class to
// inherit from more than one parent class.  Thus it is
// impossible for a class such as this one, RunnableSquare,
// to be both a Square and a Thread.  But Java allows a
// given class to implement as many interfaces as one wants,
// and this is the key to making class RunnableSquare both
// a Square and a Thread-like object.

// The Runnable interface simply requires that its implementor
// have a run() method declared.  The run() method is where
// code that will run in a separate Thread, such as animation
// code, goes.  The Thread class includes a constructor that
// takes as its parameter an object implementing the Runnable
// interface, such as a RunnableSquare object.  If one includes 
// code in an application similar to
//
//     RunnableSquare mySquare = new RunnableSquare(...);
//     Thread myThread = new Thread(mySquare);
//     myThread.start();
//
// then, mirabile dictu, the run method of mySquare will be
// invoked and run in the Thread myThread.  

// A RunnableSquare object should know
//
//   1. Everything a MoveableSquare object knows.
//   2. Its parent object, eg. the Frame or Applet Panel on which 
//        the animation will be displayed (state variable).
//   3. The Graphics and Image buffer objects it is going to use
//        for background drawing (state variables).
//   4. How fast to move (state variable).
//   5. How to set and return the parent object (methods).
//   6. How to set and return the Image buffer object (methods).
//   7. How to set and return the pause time variable to control 
//        its speed of movement (methods).
//   8. How to run itself (method -- all objects implementing 
//        the Runnable interface Threads have to have a run
//        run method).  Here is where the animation will actually
//        be done.

import java.awt.*;
import MoveableSquare;

public class RunnableSquare extends MoveableSquare
                            implements Runnable {

  // all the Square object state variables are
  // automatically inherited.  No need to repeat
  // ourselves.

  // all the Square methods are inhereted
  // too, so we don't need to repeat them.

  // Additional object state variables and methods
  // unique to this class:

   private Component parent;  // the Canvas, Frame or Applet Panel
                              // on which we draw the animation

   private Graphics gBuffer;   // for double buffering, as usual,
   private Image imageBuffer;  // but this time we will pass them
                               // in via the Constructor

   private Graphics g;  // our parent's Graphics object, on which 
                        // we draw

   private int pauseTime;  // in milliseconds.  The larger this
                           // is, the more slowly we move.

   // Constructor: as usual, just initializes variables.  The
   // parameters are:
     //  int x, int y     : the x and y coordinates of our 
     //                     our position vector
     //  int size         : our width = our height
     //  int rx, ry       : the x and y components of our 
     //                     heading vector.
     //  int d            : the distance we move (in multiples 
     //                     of our heading vector length).
     //  Color color      : our Color                           
     //  Component parent  : the Canvas, Frame or Applet Panel
     //                      on which we draw the animation
     //  Image imageBuffer : the Image buffer object we use for
     //                      background drawing
     //  int pauseTime    : our pauseTime in milliseconds
   public RunnableSquare(int x, int y, int size,
                         int rx, int ry, int d, Color color,
                         Component parent, 
                         Image imageBuffer,
                         int pauseTime) {

     super(x, y, size, rx, ry , d, color);  
                                // a call to super() invokes the
                                // the parent class constructor,
                                // so we don't have to repeat
                                // ourselves

     // now deal with the stuff unique to RunnableSquares

     this.parent = parent;

     // we can initialize the imageBuffer and gBuffer variables
     // here in the constructor, rather than doing it in init(),
     // because THEY already exist, even if we don't.
     this.imageBuffer = imageBuffer;
     this.gBuffer = imageBuffer.getGraphics();

     this.pauseTime = pauseTime;

   } // Constructor RunnableSquare

   // init(): initializes the Graphics object we draw on.  
   //         Remember, we can't do this in our Constructor, 
   //         because we have to exist before a call to 
   //         parent.getGraphics() will work.
   //
   //         This is an example of an "internal method".  It will
   //         be called from run().  Therefore, it is declared to 
   //         be private.
   private void init() {

     // initialize g to the Graphics object (context) of our parent.
     // g is now the g of our parent's paint(Graphics g) method.
     g = parent.getGraphics();

     // no need to call repaint() because we are not going to
     // do our drawing in paint()!

   } // init()

   // set our parent object
   public void setParent(Component parent) {

      this.parent = parent;

   } // setParent()

   // return our Parent object
   public Component getParent() {

      return parent;

   } // getParent()

   // set our Image buffer object
   public void setImageBuffer(Image imageBuffer) {

     this.imageBuffer = imageBuffer;

   } // setImageBuffer()

   // return our Image buffer object
   public Image getImageBuffer(Image imageBuffer) {

      return imageBuffer;

   } // getImageBuffer()

   // set our pauseTime
   public void setPauseTime(int pauseTime) {

      this.pauseTime = pauseTime;

   } // setPauseTime()

   // return our pauseTime 
   public int getPauseTime(int pauseTime) {

      return pauseTime;

   } // getPauseTime()

   // Here is where we do our assigned task, animate the Square!
   // All Runnable objects have to have a run method.
   public void run() {

      Color bg = parent.getBackground();  // background color

      init();  // initialize Graphics and Image buffers

      // This is interesting.  For the first time, we are going to
      // do our drawing outside of the paint() method.  This is so
      // we can continually draw in one Thread of execution while
      // our creator application (a Frame or an Applet) does other
      // things like event processing.  Also, we don't have a 
      // paint() method because we are not a Frame or Applet Panel.

      // Here comes the animation!
      for(int i = 1; i <= d; i++) {

         // move -- do drawing offscreen on the Graphics
         //         buffer object
         draw(gBuffer, bg);  // hide ourself
         move(rx, ry);
         draw(gBuffer);

         //draw completed image on the screen
         g.drawImage(imageBuffer, 0, 0, parent);

         // To pause, so the animation doesn't go too fast, use
         // the sleep() method of the Thread class.  Unfortunately,
         // it throws an InterruptedException, so we have to embed
         // our simple pause statement in a try -- catch clause.
         try {
           Thread.sleep(pauseTime);  // pauseTime ms pause
         }
         catch (InterruptedException e) {
           // do nothing
         }
       }

   } // run()

} // class RunnableSquare

   